@@ -7,10 +7,12 @@ use Config;
 use FFI::Platypus;
 use FFI::Platypus::Memory qw( malloc free );
 use Carp qw( croak carp );
-use File::ShareDir ();
+use File::Spec;
+use File::ShareDir::Dist qw( dist_share );
 
 # ABSTRACT: Tiny C Compiler for FFI
-our $VERSION = '0.17_02'; # VERSION
+our $VERSION = '0.27_01'; # TRIAL VERSION
+$VERSION = eval $VERSION;
 
 
 sub _dlext ()
@@ -20,17 +22,8 @@ sub _dlext ()
 
 our $ffi = FFI::Platypus->new;
 $ffi->lib(
-  $ENV{FFI_TINYCC_LIBTCC_SO} || (eval { File::ShareDir::dist_dir('FFI-TinyCC') } ? File::ShareDir::dist_file('FFI-TinyCC', "libtcc." . _dlext) : do {
-    require Path::Class::File;
-    Path::Class::File
-      ->new($INC{'FFI/TinyCC.pm'})
-      ->dir
-      ->parent
-      ->parent
-      ->file('share', 'libtcc.' . _dlext)
-      ->stringify
-  }
-));
+  File::Spec->catfile(dist_share( 'FFI-TinyCC' ), 'libtcc.' . _dlext)
+);
 
 $ffi->custom_type( tcc_t => {
   perl_to_native => sub {
@@ -157,20 +150,7 @@ sub add_symbol
 {
   my($self, $name, $ptr) = @_;
   my $r;
-  if(ref($ptr) && eval { $ptr->isa('FFI::Raw::Callback') })
-  {
-    require FFI::Raw;
-    my($lib) = $ffi->lib;
-    my $add_symbol = FFI::Raw->new($lib, 'tcc_add_symbol',
-      FFI::Raw::int(),
-      FFI::Raw::ptr(), FFI::Raw::str(), FFI::Raw::ptr(),
-    );
-    $r = $add_symbol->call($self->{handle}, $name, $ptr);
-  }
-  else
-  {
-    $r = _add_symbol($self, $name, $ptr);
-  }
+  $r = _add_symbol($self, $name, $ptr);
   die FFI::TinyCC::Exception->new($self) if $r == -1;
   $self;
 }
@@ -179,17 +159,20 @@ sub add_symbol
 sub detect_sysinclude_path
 {
   my($self) = @_;
+  
+  my @path_list;
+  
   if($^O eq 'MSWin32')
   {
     require File::Spec;
-    $self->add_sysinclude_path(File::Spec->catdir(File::ShareDir::dist_dir('Alien-TinyCC'), 'include'));
+    push @path_list, File::Spec->catdir(dist_share('Alien-TinyCC'), 'include');
   }
   elsif($Config{incpth})
   {
     require Alien::TinyCC;
     require File::Spec;
-    $self->add_sysinclude_path(File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include )));
-    $self->add_sysinclude_path($_) for split /\s+/, $Config{incpth};
+    push @path_list, File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include ));
+    push @path_list, split /\s+/, $Config{incpth};
   }
   elsif($Config{ccname} eq 'gcc')
   {
@@ -208,14 +191,20 @@ sub detect_sysinclude_path
     
     require Alien::TinyCC;
     require File::Spec;
-    $self->add_sysinclude_path(File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include )));
     
-    $self->add_sysinclude_path($_) for map { chomp; s/^ //; $_ } @lines;
+    push @path_list, File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include ));
+    push @path_list, map { chomp; s/^ //; $_ } @lines;
   }    
   else
   {
     croak "Cannot detect sysinclude path";
   }
+  
+  croak "Cannot detect sysinclude path" unless grep { -d $_ } @path_list;
+  
+  $self->add_sysinclude_path($_) for @path_list;
+  
+  @path_list;
 }
 
 
@@ -270,7 +259,7 @@ sub get_symbol
     my $size = _relocate($self, undef);
     $self->{store} = malloc($size);
     my $r = _relocate($self, $self->{store});
-    FFI::TinyCC::Exception->new($self) if $r == -1;
+    die FFI::TinyCC::Exception->new($self) if $r == -1;
     $self->{relocate} = 1;
   }
   _get_symbol($self, $symbol_name);
@@ -279,22 +268,6 @@ sub get_symbol
 
 _method output_file => qw( string );
 
-
-# this variable will too be removed once this module
-# is ported to FFI::Platypus, so do not depeend on it!
-our $_get_ffi_raw_deprecation = 1;
-
-sub get_ffi_raw
-{
-  carp "FFI::TinyCC->get_ffi_raw is deprecated" if $_get_ffi_raw_deprecation;
-  my($self, $symbol, @types) = @_;
-  croak "you must at least specify a return type" unless @types > 0;
-  my $ptr = $self->get_symbol($symbol);
-  croak "$symbol not found" unless $ptr;
-  require FFI::Raw;
-  FFI::Raw->new_from_ptr($self->get_symbol($symbol), @types);
-}
-
 package
   FFI::TinyCC::Exception;
 
@@ -364,12 +337,12 @@ FFI::TinyCC - Tiny C Compiler for FFI
 
 =head1 VERSION
 
-version 0.17_02
+version 0.27_01
 
 =head1 SYNOPSIS
 
  use FFI::TinyCC;
- use FFI::Platypus::Declare qw( int );
+ use FFI::Platypus;
  
  my $tcc = FFI::TinyCC->new;
  
@@ -382,10 +355,36 @@ version 0.17_02
  });
  
  my $address = $tcc->get_symbol('find_square');
- attach [$address => 'find_square'] => [int] => int;
+ my $ffi = FFI::Platypus->new;
+ $ffi->attach([$address => 'find_square'] => ['int'] => 'int');
  
  print find_square(4), "\n"; # prints 16
 
+For code that requires system headers:
+
+ use FFI::TinyCC;
+ use FFI::Platypus;
+ 
+ my $tcc = FFI::TinyCC->new;
+ 
+ # this will throw an exception if the system
+ # include paths cannot be detected.
+ $tcc->detect_sysinclude_path;
+ 
+ $tcc->compile_string(q{
+   #include <stdio.h>
+   
+   void print_hello()
+   {
+     puts("hello world");
+   }
+ });
+ 
+ my $address = $tcc->get_symbol('print_hello');
+ my $ffi = FFI::Platypus->new;
+ $ffi->attach([$address => 'print_hello'] => [] => 'void');
+ print_hello();
+
 =head1 DESCRIPTION
 
 This module provides an interface to a very small C compiler known as 
@@ -443,15 +442,13 @@ Compile a string containing C source code.
 Add the given given symbol name / callback or pointer combination. See 
 example below for how to use this to call Perl from Tiny C code.
 
-It will accept a L<FFI::Raw::Callback> at a performance penalty. If 
-possible pass in the pointer to the C entry point instead.
+If you are using L<FFI::Platypus> you can use L<FFI::Platypus#cast>
+to get a pointer to a closure:
 
-If you are using L<FFI::Platypus> you can use L<FFI::Platypus#cast> or 
-L<FFI::Platypus::Declare#cast> to get a pointer to a closure:
-
- use FFI::Platypus::Declare;
- my $clousre = closure { return $_[0]+1 };
- my $pointer = cast '(int)->int' => 'opaque', $closure;
+ use FFI::Platypus;
+ my $ffi = FFI::Platypus;
+ my $closure = $ffi->closure(sub { return $_[0]+1 });
+ my $pointer = $ffi->cast('(int)->int' => 'opaque', $closure);
  
  $tcc->add_symbol('foo' => $pointer);
 
@@ -463,10 +460,14 @@ L<FFI::Platypus::Declare#cast> to get a pointer to a closure:
 
  $tcc->detect_sysinclude_path;
 
-Attempt to find and configure the appropriate system include files. If 
+Attempt to find and configure the appropriate system include directories. If 
 the platform that you are on does not (yet?) support this functionality 
 then this method will throw an exception.
 
+[version 0.19]
+
+Returns the list of directories added to the system include directories.
+
 =head3 add_include_path
 
  $tcc->add_include_path($path);
@@ -511,7 +512,7 @@ Undefine the given symbol.
 Set the output type.  This must be called before any compilation.
 
 Output formats may not be supported on your platform.  C<exe> is
-NOT supported on *BSD or OS X.
+NOT supported on *BSD or OS X.  It may NOT be supported on Linux.
 
 As a basic baseline at least C<memory> should be supported.
 
@@ -539,8 +540,7 @@ Add the given directory to the search path used to find libraries.
 
 Return symbol address or undef if not found.  This can be passed into 
 the L<FFI::Platypus#function> method, L<FFI::Platypus#attach> method, 
-L<FFI::Platypus::Declare#function> function or similar interface that 
-takes a pointer to a C function.
+or similar interface that takes a pointer to a C function.
 
 =head3 output_file
 
@@ -550,34 +550,6 @@ Output the generated code (either executable, object or DLL) to the
 given filename. The type of output is specified by the 
 L<set_output_type|/set_output_type> method.
 
-=head3 get_ffi_raw
-
-B<DEPRECATED>
-
- my $ffi = $tcc->get_ffi_raw($symbol_name, $return_type, @argument_types);
-
-Given the name of a function, return an L<FFI::Raw> instance that will 
-allow you to call it from Perl.
-
-This method is deprecated, and will be removed from a future version of 
-L<FFI::TinyCC>, but not before January 31, 2017.  It will issue a 
-warning if you try to use it.  Instead of this:
-
- my $function = $ffi->get_ffi_raw($name, FFI::void);
- $function->();
-
-Do this:
-
- use FFI::Raw;
- my $function = FFI::Raw->new_from_ptr($ffi->get_symbol($name), FFI::void);
- $function->();
-
-Or better yet, use L<FFI::Platypus> instead:
-
- use FFI::Platypus::Declare;
- attach [$ffi->get_symbol($name) => 'function'] => [] => 'void';
- function();
-
 =head1 EXAMPLES
 
 =head2 Calling Tiny C code from Perl
@@ -601,10 +573,11 @@ Or better yet, use L<FFI::Platypus> instead:
 =head2 Calling Perl from Tiny C code
 
  use FFI::TinyCC;
- use FFI::Platypus::Declare qw( opaque );
+ use FFI::Platypus;
  
- my $say = closure { print $_[0], "\n" };
- my $ptr = cast '(string)->void' => opaque => $say;
+ my $ffi = FFI::Platypus->new;
+ my $say = $ffi->closure(sub { print $_[0], "\n" });
+ my $ptr = $ffi->cast('(string)->void' => 'opaque' => $say);
  
  my $tcc = FFI::TinyCC->new;
  $tcc->add_symbol(say => $ptr);
@@ -630,7 +603,7 @@ Or better yet, use L<FFI::Platypus> instead:
 =head2 Attaching as a FFI::Platypus function from a Tiny C function
 
  use FFI::TinyCC;
- use FFI::Platypus::Declare qw( int );
+ use FFI::Platypus;
  
  my $tcc = FFI::TinyCC->new;
  
@@ -647,7 +620,8 @@ Or better yet, use L<FFI::Platypus> instead:
  
  my $address = $tcc->get_symbol('calculate_square');
  
- attach [$address => 'square'] => [int] => int;
+ my $ffi = FFI::Platypus->new;
+ $ffi->attach([$address => 'square'] => ['int'] => 'int');
  
  print square($value), "\n";
 
@@ -672,6 +646,8 @@ with L<Alien::TinyCC>.
 
 =item L<FFI::Platypus>
 
+=item L<C::Blocks>
+
 =item L<Alien::TinyCC>
 
 =item L<C::TinyCompiler>
@@ -686,18 +662,24 @@ from L<Inline::C::Parser::RegExp>
 
 The license details for the parser are:
 
-Copyright 2002 Brian Ingerson
-Copyright 2008, 2010-2012 Sisyphus
-Copyright 2013 Steffen Muellero
+ Copyright 2002 Brian Ingerson
+ Copyright 2008, 2010-2012 Sisyphus
+ Copyright 2013 Steffen Muellero
 
 This program is free software; you can redistribute it and/or modify it 
 under the same terms as Perl itself.
 
-=cut
-
 =head1 AUTHOR
 
-Graham Ollis <plicease@cpan.org>
+Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
+
+Contributors:
+
+aero
+
+Dylan Cali (calid)
+
+pipcet
 
 =head1 COPYRIGHT AND LICENSE
 
